/* slld_hal_example.c - SLLD Hardware Abstraction Layer example Code */

/**************************************************************************
 * Copyright 2011 Spansion LLC. All Rights Reserved.
 *
 * This software is owned and published by:
 * Spansion LLC, 915 DeGuigne Drive, Sunnyvale, CA 94088 ("Spansion").
 *
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
 * BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
 *
 * This software constitutes driver source code for use in programming Spansion's
 * Flash memory components. This software is licensed by Spansion to be adapted only
 * for use in systems utilizing Spansion's Flash memories. Spansion is not be
 * responsible for misuse or illegal use of this software for devices not
 * supported herein.  Spansion is providing this source code "AS IS" and will
 * not be responsible for issues arising from incorrect user implementation
 * of the source code herein.
 *
 * Spansion MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
 * REGARDING THE SOFTWARE, ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED
 * USE, INCLUDING, WITHOUT LIMITATION, NO IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A  PARTICULAR PURPOSE OR USE, OR NONINFRINGEMENT.  Spansion WILL
 * HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, NEGLIGENCE OR
 * OTHERWISE) FOR ANY DAMAGES ARISING FROM USE OR INABILITY TO USE THE SOFTWARE,
 * INCLUDING, WITHOUT LIMITATION, ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, SAVINGS OR PROFITS,
 * EVEN IF Spansion HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software may be replicated in part or whole for the licensed use,
 * with the restriction that this Copyright notice must be included with
 * this software, whether used in part or whole, at all times.
 */

#include <stdio.h>
#include "slld_targetspecific.h"
#include "slld.h"
#include "slld_hal.h"
#include "typedef.h"

/* Buffers for SPI transaction */
static DWORD g_buf_tx[CONTROLLER_BUFFER_SIZE + 4u];
static DWORD g_buf_rx[CONTROLLER_BUFFER_SIZE + 4u];

// ***************************************************************************
//  FLASH_READ - HAL read function
//
//  input : command               write a single command byte to flash
//          sys_addr              system address to be used
//          data_buffer           Pointer to the data buffer where to store the read data
//          Number_Of_Read_Bytes  number of bytes to be read//
//
//  return value : status of the operation - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_READ (BYTE command,
                        ADDRESS sys_addr,
                        BYTE* data_buffer,
                        int Number_Of_Read_Bytes)
{
    SLLD_STATUS status    = SLLD_OK;
    int         len_dummy = 0;
    int         len_hdr   = 0;
    int         cnt;

    /* Write the command */
    g_buf_tx[0] = command;
    len_hdr++;

    /* Write the address */
    if (sys_addr != ADDRESS_NOT_USED)
    {
        switch (command)
        {
            case SPI_DUALIO_HPRD_CMD:
            case SPI_DUALIO_HPRD_4B_CMD:

                /* Originally it writes to SPIDATA_D, which is not supported */
                status = SLLD_E_HAL_ERROR;
                break;

            case SPI_QUADIO_HPRD_CMD:
            case SPI_QUADIO_HPRD_4B_CMD:

                /* Originally it writes to SPIDATA_Q, which is not supported */
                status = SLLD_E_HAL_ERROR;
                break;

            default:
                g_buf_tx[len_hdr + 0]  = (BYTE)((sys_addr >> 16) & 0x000000FF);
                g_buf_tx[len_hdr + 1u] = (BYTE)((sys_addr >> 8) & 0x000000FF);
                g_buf_tx[len_hdr + 2u] = (BYTE)(sys_addr & 0x000000FF);
                len_hdr               += 3u;
                break;
        }
    }

    /* Write the dummy bytes */
    switch (command)
    {
        case SPI_FAST_READ_CMD:
        case SPI_FAST_READ_4B_CMD:
        case SPI_DUALIO_RD_CMD:
        case SPI_DUALIO_RD_4B_CMD:
        case SPI_QUADIO_RD_CMD:
        case SPI_QUADIO_RD_4B_CMD:
        case SPI_OTPR_CMD:

            /* Write a dummy byte to the data bus */
            len_dummy = 1u;

            for (cnt = 0 ; cnt < len_dummy ; cnt++)
            {
                g_buf_tx[len_hdr + cnt] = 0x0;
            }

            len_hdr += len_dummy;

            break;

        case SPI_DUALIO_HPRD_CMD:
        case SPI_DUALIO_HPRD_4B_CMD:

            /* Originally it writes to SPIDATA_D, which is not supported */
            status = SLLD_E_HAL_ERROR;
            break;

        case SPI_QUADIO_HPRD_CMD:
        case SPI_QUADIO_HPRD_4B_CMD:

            /* Originally it writes to SPIDATA_Q, which is not supported */
            status = SLLD_E_HAL_ERROR;
            break;

        case SPI_RES_CMD:

            /* Write the dummy bytes to the data bus */
            len_dummy = 3u;

            for (cnt = 0 ; cnt < len_dummy ; cnt++)
            {
                g_buf_tx[len_hdr + cnt] = 0x0;
            }

            len_hdr += len_dummy;

            break;

        default:
            len_dummy = 0;
            break;
    }

    /* Read the data */
    if (Number_Of_Read_Bytes != 0)
    {
        switch (command)
        {
            case SPI_DUALIO_RD_CMD:
            case SPI_DUALIO_RD_4B_CMD:
            case SPI_DUALIO_HPRD_CMD:
            case SPI_DUALIO_HPRD_4B_CMD:

                /* Originally it reads from SPIDATA_D, which is not supported */
                status = SLLD_E_HAL_ERROR;
                break;

            case SPI_QUADIO_RD_CMD:
            case SPI_QUADIO_RD_4B_CMD:
            case SPI_QUADIO_HPRD_CMD:
            case SPI_QUADIO_HPRD_4B_CMD:

                /* Originally it reads from SPIDATA_Q, which is not supported */
                status = SLLD_E_HAL_ERROR;
                break;

            default:

                /* No action */
                break;
        }
    }

    /* Safety check */
    if (((len_hdr + Number_Of_Read_Bytes) > sizeof(g_buf_rx)) ||
        ((len_hdr + Number_Of_Read_Bytes) > sizeof(g_buf_tx)))
    {
        /* Data will not fit in buffer */
        status = SLLD_E_HAL_ERROR;
    }

    /* Safety check */
    if (SLLD_OK == status)
    {
        /* Perform SPI transaction, which sends header and reads the data */
        if (bsp_spi_transaction_blocking(g_buf_tx, g_buf_rx, len_hdr + Number_Of_Read_Bytes) != RESULT_SUCCESS)
        {
            status = SLLD_E_HAL_ERROR;
        }
        else
        {
            /* Copy from transaction buffer into output */
            for (cnt = 0 ; cnt < Number_Of_Read_Bytes ; cnt++)
            {
                data_buffer[cnt] = g_buf_rx[len_hdr + cnt];
            }
        }
    }

    return status;
}

// ***************************************************************************
//  FLASH_WRITE - HAL write function
//
//  input : command                  write a single command byte to flash
//          sys_addr                 system address to be used
//          data_buffer              Pointer to the data buffer where to store the written data
//          Number_Of_Written_Bytes  number of bytes to be written
//
//  return value : status of the operation - FAIL or SUCCESS
// ***************************************************************************
SLLD_STATUS FLASH_WRITE (BYTE command,
                         ADDRESS sys_addr,
                         BYTE* data_buffer,
                         int Number_Of_Written_Bytes)
{
    SLLD_STATUS status  = SLLD_OK;
    int         len_hdr = 0;
    int         cnt;

    /* Write the command */
    g_buf_tx[0] = command;
    len_hdr++;

    /* Write the address */
    if (sys_addr != ADDRESS_NOT_USED)
    {
        g_buf_tx[len_hdr + 0]  = (BYTE)((sys_addr >> 16) & 0x000000FF);
        g_buf_tx[len_hdr + 1u] = (BYTE)((sys_addr >> 8) & 0x000000FF);
        g_buf_tx[len_hdr + 2u] = (BYTE)(sys_addr & 0x000000FF);
        len_hdr               += 3u;
    }

    /* Write the data */
    if (Number_Of_Written_Bytes != 0)
    {
        switch (command)
        {
            case SPI_QPP_CMD:
            case SPI_QPP_4B_CMD:

                /* Originally it writes to SPIDATA_Q, which is not supported */
                status = SLLD_E_HAL_ERROR;
                break;

            default:

                /* No action */
                break;
        }
    }

    /* Safety check */
    if (((len_hdr + Number_Of_Written_Bytes) > sizeof(g_buf_rx)) ||
        ((len_hdr + Number_Of_Written_Bytes) > sizeof(g_buf_tx)))
    {
        /* Data will not fit in buffer */
        status = SLLD_E_HAL_ERROR;
    }

    /* Safety check */
    if (SLLD_OK == status)
    {
        /* Perform SPI transaction, which sends header and writes the data */
        if (bsp_spi_transaction_blocking(g_buf_tx, g_buf_rx, len_hdr + Number_Of_Written_Bytes) != RESULT_SUCCESS)
        {
            status = SLLD_E_HAL_ERROR;
        }
    }

    return status;
}

/*****************************************************************************/
